#!/usr/bin/python

import subprocess
import sys, getopt, os, time, dbus, json
from dbus.mainloop.glib import DBusGMainLoop

DBUS_INTERFACE_PROPERTIES='org.freedesktop.DBus.Properties'
MM_DBUS_SERVICE='org.freedesktop.ModemManager1'
MM_DBUS_PATH='/org/freedesktop/ModemManager1'
MM_DBUS_OBJECT_MANAGER='org.freedesktop.DBus.ObjectManager'
MM_DBUS_INTERFACE_MODEM='org.freedesktop.ModemManager1.Modem'
MM_DBUS_INTERFACE_MODEM_SIGNAL='org.freedesktop.ModemManager1.Modem.Signal'
MM_DBUS_INTERFACE_SIM='org.freedesktop.ModemManager1.Sim'

MM_MODEM_STATE_FAILED=-1
MM_MODEM_STATE_UNKNOWN=0
MM_MODEM_STATE_INITIALIZING=1
MM_MODEM_STATE_LOCKED=2
MM_MODEM_STATE_DISABLED=3
MM_MODEM_STATE_DISABLING=4
MM_MODEM_STATE_ENABLING=5
MM_MODEM_STATE_ENABLED=6
MM_MODEM_STATE_SEARCHING=7
MM_MODEM_STATE_REGISTERED=8
MM_MODEM_STATE_DISCONNECTING=9
MM_MODEM_STATE_CONNECTING=10
MM_MODEM_STATE_CONNECTED=11

MM_MODEM_STATE_FAILED_REASON_NONE=0
MM_MODEM_STATE_FAILED_REASON_UNKNOWN=1
MM_MODEM_STATE_FAILED_REASON_SIM_MISSING=2
MM_MODEM_STATE_FAILED_REASON_SIM_ERROR=3

DEFAULT_RETRIES=10
SKIP_ANTENNA_TEST=0

antenna_primary=-255
antenna_diversity=-255

def load_config(path):
    if os.path.isfile(path):
        with open(path) as data_file:
            data = json.load(data_file)
            return data
    return None

def run_modem_test():
    global antenna_primary
    global antenna_diversity
    modem_scan_count=DEFAULT_RETRIES
    modem_found=0
    modem_pass=0
    sim_found=0
    modem_test_mode_reset=0

    while  modem_scan_count > 0:
        if (modem_scan_count == 1) and (modem_test_mode_reset == 0):
            print("Last scan")
            if (modem_found == 0) and (os.path.exists("/dev/celltestmode") == True):
                print("Modem left in test mode, resetting and starting over")
                result = subprocess.check_output("/usr/sbin/celltest /dev/celltestmode", shell=True)
                print result
                time.sleep(3)
                modem_scan_count=DEFAULT_RETRIES
                modem_test_mode_reset=1

        print("\tScanning for modems...")
        message="" 

        try:            
            bus=dbus.SystemBus()
            mgr_proxy = bus.get_object(MM_DBUS_SERVICE, MM_DBUS_PATH)
            mgr_iface = dbus.Interface(mgr_proxy, MM_DBUS_OBJECT_MANAGER)  
            modems = mgr_iface.GetManagedObjects()

            for m in modems:
                print("\tModems Found")
                proxy = bus.get_object(MM_DBUS_SERVICE, m)
		#print("modem:" + m) 
                # Properties
                props_iface = dbus.Interface(proxy, dbus_interface=DBUS_INTERFACE_PROPERTIES)

                model = props_iface.Get(MM_DBUS_INTERFACE_MODEM, 'Model')
                rev = props_iface.Get(MM_DBUS_INTERFACE_MODEM, 'Revision')
                state = props_iface.Get(MM_DBUS_INTERFACE_MODEM, 'State')
                state_reason = props_iface.Get(MM_DBUS_INTERFACE_MODEM, 'StateFailedReason')
                signal_quality = props_iface.Get(MM_DBUS_INTERFACE_MODEM, 'SignalQuality')
        
                message="\t" + model + " " + rev + "\r\n"
              #  print("\t" + model + " " + rev)
                message=message + "\tModem Detected\r\n"
              #  print("\tModem Detected")
              #  print("\tSignalQuality: " + str(signal_quality[0]))
              #  print("\tPC05 Cellular Selftest: PASSED") 
                modem_found= modem_found + 1
                if (model == "LE910-SVG") or (model == "HE910") or (model == "LE910-EUG") or (model == "LE910-EU V2") or (model == "LE910C4-NF"):
                    #print("\tModel: " + model)
                    if (state == MM_MODEM_STATE_FAILED):
                        if (state_reason == MM_MODEM_STATE_FAILED_REASON_SIM_MISSING):
                            #print("\tNo SIM Detected")
                            message=message + "\tNo SIM Detected" + "\r\n"
                        if (state_reason == MM_MODEM_STATE_FAILED_REASON_SIM_ERROR):
                            #print("\tSIM ERROR")
                            message=message + "\tSIM ERROR" + "\r\n"
                    if (state >= MM_MODEM_STATE_ENABLED and state <= MM_MODEM_STATE_CONNECTED):
 
                        #heck if SIM is available
                        simPath = props_iface.Get(MM_DBUS_INTERFACE_MODEM, 'Sim')
                        simProxy = bus.get_object(MM_DBUS_SERVICE, simPath)
                        simProps_iface = dbus.Interface(simProxy, dbus_interface=DBUS_INTERFACE_PROPERTIES)
                        simID = simProps_iface.Get(MM_DBUS_INTERFACE_SIM, 'SimIdentifier')
                        if len(simID) > 10:
                            sim_found=1

                        message=message + "\tSIMID: " + simID + "\r\n"
 
                        if(SKIP_ANTENNA_TEST == 1):
                            message=message + "\tPC05 Detected - Skipping Antenna test\r\n"
                        else:
                            if(model == "HE910") or (model == "LE910-SVG"): #3G modem doesn't support extended signal_quality, read it from AT command, treat verizon like this for now to deal with SIM cards in a funky state
                                modem_iface = dbus.Interface(proxy, dbus_interface=MM_DBUS_INTERFACE_MODEM)
                                retry = DEFAULT_RETRIES
                                rssi="99"
                                while(retry > 0 and int(rssi) == 99):
                                    csq=modem_iface.Command("AT+CSQ", 5)
                                    print (csq)
                                    csq=csq.replace(": ", ",")
                                    id,rssi,e = csq.split(",")  
                                    if(int(rssi) == 99):
                                        time.sleep(1) 
                                    retry = retry - 1
 
                                if ( int(rssi) > 10 and int(rssi) < 50):                                 
                                    message=message + "\tSignalQuality: " + str(int(rssi)*3) + "\r\n"
                                    modem_pass=1
                                else:
                                    message=message + "\tError: Antenna failure" + "\r\n"
    
                            else:	
                                retry = DEFAULT_RETRIES
                                if(model == "LE910-EU V2"):
                                   print("Putting device in test mode....")
                                   #time.sleep(5) #if we try to put modem in test mode too fast it will reset itself...
                                   modem_iface = dbus.Interface(proxy, dbus_interface=MM_DBUS_INTERFACE_MODEM)
                                   resp=modem_iface.Command("AT#TESTMODE?", 10)
                                   print ("." + resp)
                                   time.sleep(1)
                                   resp=modem_iface.Command("AT#TESTMODE=\"TM\"", 30)
                                   print ("." + resp)
                                   time.sleep(4)
                                   #modem resets at this point... what is a good way of finding the ACM port??

                                   #execute factoryTX on pc attached to sdr
                                   #./factoryTX -f 927000000 -r 20
                                   timeout = 10
                                   print("Waiting for modem to come online in test mode")
                                   while timeout > 0:
                                       if os.path.exists("/dev/celltestmode") == True:
                                           print("Modem online...")
                                           time.sleep(2) #need to wait until the +PBREADY message comes out
                                           timeout = 0
                                       else:
                                           print(".")
                                           timeout = timeout - 1
                                           time.sleep(2)

                                   if os.path.exists("/dev/celltestmode") == True:
                                       print("Running test...")
                                       result = subprocess.check_output("/usr/sbin/celltest /dev/celltestmode", shell=True)
                                       print("Parsing results...")
                                       for line in result.splitlines():
                                           print(line)
                                           if line.find("dBm") != -1:
                                              reading=line.split()
                                              if(antenna_primary == -255):
                                                  antenna_primary=reading[0]
                                                  print("Primary: " + antenna_primary)
                                              else:
                                                  antenna_diversity=reading[0]
                                                  print("Diversity: " + antenna_diversity)

                                       message=message +  "\tPrimary/Diversity: " + antenna_primary + " " + antenna_diversity

                                       if(int(antenna_primary) > -50) and (int(antenna_diversity) > -50):
                                           modem_pass = 1
                                       else:
                                           modem_pass = 0
                                           modem_scan_count=0
                                   else:
                                       print("Modem never entered testmode...")
                                       modem_pass = 0
                                       modem_scan_count=0
                                elif(model == "LE910-EUG"): 
                                   print("Putting device in test mode....")
                                   #time.sleep(5) #if we try to put modem in test mode too fast it will reset itself...
                                   modem_iface = dbus.Interface(proxy, dbus_interface=MM_DBUS_INTERFACE_MODEM)

                                   resp=modem_iface.Command("ATI4", 10)
                                   print ("." + resp)
                                   time.sleep(1)
                                   resp=modem_iface.Command("AT#TESTMODE?", 10)
                                   print ("." + resp)
                                   time.sleep(1)
                                   resp=modem_iface.Command("AT#TESTMODE=\"TM\"", 30)
                                   print ("." + resp)
                                   time.sleep(1)
                                   resp=modem_iface.Command("AT#TESTMODE=\"SETGSMBAND 1\"", 15)
                                   print ("." + resp)
                                   time.sleep(1)
                                   resp=modem_iface.Command("AT#TESTMODE=\"TCH\"", 15)
                                   print ("." + resp)
                                   time.sleep(1)
                                   resp=modem_iface.Command("AT#TESTMODE=\"PL 29\"", 15)
                                   print ("." + resp)
                                   time.sleep(1)  
                                   resp=modem_iface.Command("AT#TESTMODE?", 10)
                                   print ("." + resp)
                                   time.sleep(1)
                                       
                                   message=message +  "Modem in Continuous TX mode - validate with Spectrum Analyzer"
                                   
                                   modem_pass = 1
                                else:
                                    #default test is to check extended signal quality
                                    while(retry > 0 and str(signal_quality) == "0"):
                                        signal_quality = props_iface.Get(MM_DBUS_INTERFACE_MODEM, 'SignalQuality')
                                        time.sleep(1) 
                                        retry = retry - 1
                            
                                    message=message + "\tSignalQuality: " + str(signal_quality[0]) + "\r\n"
                                    if (str(signal_quality[0]) == "0"):
                                        message=message + "\tError: Antenna failure" + "\r\n"
                                    else:
                                        modem_pass=1

        except Exception, e:
            print "\tDbus Error : %s" % e

  
        if(modem_scan_count > 0):
             modem_scan_count = modem_scan_count - 1
             if (modem_found > 1 or modem_pass == 1):
                 print( message )

                 if(sim_found == 1):
                     print("\tPC05 Cellular Selftest: PASSED") 
                     #os.system("echo 'PC05 PASSED' >> /home/root/celltest.log")
                 else: 
                     print("\tPC05 Cellular Selftest: FAILED") 
                     #os.system("echo 'PC05 FAILED' >> /home/root/celltest.log")

                 if(modem_pass == 1):
                     print("\tPC02 Cellular Selftest: PASSED")
                     #os.system("echo 'PC02 PASSED' >> /home/root/celltest.log")
                 else:    
                     print("\tPC02 Cellular Selftest: FAILED")    
                     #os.system("echo 'PC02 FAILED' >> /home/root/celltest.log")

                 return 0
             time.sleep(5)
        else:
            print( message )
            if(modem_found == 0):
                print("\tNo modem detected")
                print("\tPC05 Cellular Selftest: FAILED")
                #os.system("echo 'PC05 FAILED' >> /home/root/celltest.log")
            if(modem_pass == 0):    
                print("\tPC02 Cellular Selftest: FAILED")    
                #os.system("echo 'PC02 FAILED' >> /home/root/celltest.log")


if len(sys.argv) < 2:
	print("Usage: Cellular.sh [A|B] [retries] [PC05]")
        sys.exit(1)

try:
    cfg = load_config("/etc/raven/cell/cellular.conf")
    if(cfg["slot"] != sys.argv[1]):
        cfg["slot"] = sys.argv[1]
        with open('/etc/raven/cell/cellular.conf', 'w') as outfile:
            json.dump(cfg, outfile)
except Exception, e:
    os.system("echo '{}' > /etc/raven/cell/cellular.conf")
    cfg = load_config("/etc/raven/cell/cellular.conf")
    cfg["slot"] = sys.argv[1]
    with open('/etc/raven/cell/cellular.conf', 'w') as outfile:
        json.dump(cfg, outfile)

if(len(sys.argv) > 2):
    DEFAULT_RETRIES = int(sys.argv[2])

if(len(sys.argv) > 3):
    if(sys.argv[3] == "PC05"):
        SKIP_ANTENNA_TEST = 1

os.system("/etc/init.d/cellular stop")
time.sleep(3)
os.system("/etc/init.d/cellular start")

#todo:: long term the "test" flag in cellular-monitor should do nothing when all models have been converted to tx mode
os.system("PN=`fw_printenv | grep rvn_bd_pn | grep 0640171390 `; if [ ${PN} ]; then killall -9 cellular-monitor; fi")

print("--Testing socket " + cfg["slot"])

run_modem_test()
