#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
from application import app

import sys
import getopt
import hashlib
import uuid
import os
import time
sys.path.append('/usr/local/bin/thrift-sgc-py')  # put gen-py files here
from ns_hwSGCCon import hwSGCConThService
from ns_hwSGCCon.ttypes import *

from thrift import Thrift
from thrift.transport import TSocket
from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol
import json
import base64

# ************************************* Defines ********************************

PORT = 9080
SOCKET = '127.0.0.1'

selectedProfile = 'Brandon1'


def WriteToJSONFile(path, fileName, data):
    if not os.path.exists(path):
        os.mkdir(path)
    filePathNameWExt = path + '/' + fileName
    with open(filePathNameWExt, 'w') as fp:
        json.dump(data, fp)


def SetProfile(name):
    transport = TSocket.TSocket(SOCKET, PORT)
    transport = TTransport.TBufferedTransport(transport)
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    client = hwSGCConThService.Client(protocol)

    print 'Setting profiel name to ' + name
    transport.open()
    client.SetProfile(name)
    transport.close()


def CreateNewProfile(
    name,
    fore_aft,
    height,
    left_right,
    ):

    transport = TSocket.TSocket(SOCKET, PORT)
    transport = TTransport.TBufferedTransport(transport)
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    client = hwSGCConThService.Client(protocol)
    position = hwSGCConThService.AntennaPosition()
    position.foreAft = fore_aft
    position.height = height
    position.leftRight = left_right
    print 'profilename ' + name
    transport.open()
    client.CreateNewMappingVehicleProfile(name, position)
    transport.close()


def EditCurrentProfile(
    name,
    fore_aft,
    height,
    left_right,
    ):

    transport = TSocket.TSocket(SOCKET, PORT)
    transport = TTransport.TBufferedTransport(transport)
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    client = hwSGCConThService.Client(protocol)
    position = hwSGCConThService.AntennaPosition()
    position.foreAft = fore_aft
    position.height = height
    position.leftRight = left_right
    print 'profilename ' + name
    transport.open()
    client.EditCurrentProfile(name, position)
    transport.close()


def SetDirection(forward):
    transport = TSocket.TSocket(SOCKET, PORT)
    transport = TTransport.TBufferedTransport(transport)
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    client = hwSGCConThService.Client(protocol)
    transport.open()
    client.SetDirectionForward(forward)
    transport.close()


def GetData(key):
    result = None
    transport = TSocket.TSocket(SOCKET, PORT)
    transport = TTransport.TBufferedTransport(transport)
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    client = hwSGCConThService.Client(protocol)
    transport.open()
    print 'Starting Request'
    if key == 'selected_profile':
        result = client.GetSelectedProfile()
        print result
    elif key == 'profile_list':
        result = client.GetListOfProfiles()
        print result
    elif key == 'antenna_position':
        result = client.GetAntennaPosition()
    elif key == 'num_of_profiles_available':
        result = client.GetNumberOfProfileSlotsAvailable()
    elif key == 'direction':
        result = client.GetIsDirectionForward()
    elif key == 'convergence_status':
        result = client.GetConvergenceStatus()
    elif key == 'source':
        source_list = GetStaticSourceList()
        index = client.GetCorrectionType()
        result = source_list[index]
    elif key == 'source_list':
        unlocks = client.GetUnlockStatus()
        result = GetSourceListFromUnlockList(unlocks.unlockList)
    transport.close()
    return result


def SetSource(source):
    index = GetSourceIndexFromList(source)
    transport = TSocket.TSocket(SOCKET, PORT)
    transport = TTransport.TBufferedTransport(transport)
    protocol = TBinaryProtocol.TBinaryProtocol(transport)
    client = hwSGCConThService.Client(protocol)
    transport.open()
    client.SetCorrectionType(int(index))
    transport.close()

def InitializeDifferentialSetting():
    source_list = GetData('source_list')
    if 'RTK' in source_list:
        SetSource('RTK')
    elif 'GS PRO' in source_list:
        SetSource('GS PRO')
    elif 'GS' in source_list:
        SetSource('GS')
    else:
        SetSource('SBAS')


def GetSourceIndexFromList(source):
    source_list = [
        'GLIDE',
        'SBAS',
        'RTK',
        'GS',
        'GS LITE',
        'GS PRO',
        ]
    source_index = 0
    for s in source_list:
        if s == source:
            break
        else:
            source_index = source_index + 1
    return source_index


def GetStaticSourceList():
    source_list = [
        'GLIDE',
        'SBAS',
        'RTK',
        'GS',
        'GS LITE',
        'GS PRO',
        ]
    return source_list

def GetSourceListFromUnlockList(unlock_list):
        source_list = [
            'GLIDE',
            'SBAS',
            ]
        if 'LITE' in unlock_list:
            source_list.append('GS LITE')

        if 'GSPRO' in unlock_list:
            source_list.append('GS PRO')
        elif 'GS' in unlock_list:
            source_list.append('GS')

        if 'RTK' in unlock_list:
            source_list.append('RTK')
        return source_list


def SetUnits(unit):
    data = {}
    data['units'] = unit
    WriteToJSONFile('/etc/raven/web-ui', 'web-ui.conf', data)


def GetUnits():
    ret_val = 'Inches' 
    if  os.path.exists('/etc/raven/web-ui/web-ui.conf'):
        with open('/etc/raven/web-ui/web-ui.conf') as json_file:
           data = json.load(json_file)
           ret_val = data['units']
    return ret_val

def ValidateAntennaHeight(height):
    ret_val = False
    if height >= 0 and height < 7000:
        ret_val = True
    return ret_val

def ValidateAntennaLeftRight(value):
    ret_val = False
    if value >= 0 and value < 32000:
        ret_val = True
    return ret_val

def IsProfileNameDuplicate(current_name,new_name):
    ret_val = False
    if current_name == new_name:
        ret_val = False
    else:
        profile_list = GetData('profile_list')
        for name in profile_list:
            if name == new_name:
                ret_val = True
    return ret_val