#!/usr/bin/python
# -*- coding: utf-8 -*-

from application import hwsgcconhandler
from application import app

from flask import Flask, render_template, session, request, \
    copy_current_request_context, redirect, url_for
from flask_wtf import FlaskForm
from wtforms import Form, TextField, validators, StringField, \
    IntegerField, SubmitField, DecimalField
from wtforms.validators import NumberRange
def ConvertFromMM(value):
    new_value = value * 0.03936996
    units = hwsgcconhandler.GetUnits()
    if units == 'Centimeters':
        new_value = value * 0.1
    return round(new_value, 1)


def ConvertToMM(value):
    new_value = value * 25.4
    units = hwsgcconhandler.GetUnits()
    if units == 'Centimeters':
        new_value = value * 10
    return new_value


class ConfigSubmitForm(FlaskForm):

    paramProfileName = StringField('Profile Name',
                                   validators=[validators.DataRequired()])
    paramHeight = DecimalField('Antenna Height',
                               validators=[NumberRange(min=0)])
    paramLeftRight = DecimalField('Antenna Left/Right',
                               validators=[NumberRange(min=-1000000)])
    submit = SubmitField('Configure')


@app.route('/')
@app.route('/index')
def index():
    return render_template('index.html')


@app.route('/login')
def login():
    return render_template('login.html')


@app.route('/mapping/edit/', methods=['GET', 'POST'])
def edit():
    result = None
    form = ConfigSubmitForm()
    result = redirect(url_for('mapping'))
    unit = hwsgcconhandler.GetUnits()
    if request.method == 'GET':
        info = hwsgcconhandler.GetData('antenna_position')
        form.paramProfileName.data = \
            hwsgcconhandler.GetData('selected_profile')
        form.paramHeight.data = ConvertFromMM(info.height)
        form.paramLeftRight.data = ConvertFromMM(info.leftRight)

    if form.validate_on_submit():
        name = form.paramProfileName.data.decode('utf-8')
        height = ConvertToMM(float(form.paramHeight.data))
        leftRight = ConvertToMM(float(form.paramLeftRight.data))
        duplicate_name = hwsgcconhandler.IsProfileNameDuplicate(hwsgcconhandler.GetData('selected_profile')
                                                                ,name)
        height_valid = hwsgcconhandler.ValidateAntennaHeight(height)
        center_offset_valid = hwsgcconhandler.ValidateAntennaLeftRight(abs(leftRight))
        if not center_offset_valid or not height_valid or duplicate_name:
            result = render_template('edit.html', cfg_form=form,
                                 Units=unit,duplicate = duplicate_name
                                 , invalid_height_offset = not height_valid
                                 , invalid_center_offset = not center_offset_valid)
        else:
            print 'Editing profile ' + name
            hwsgcconhandler.EditCurrentProfile(name, 0, height, leftRight)
    else:
        print form.errors
        print 'render edit screen'
        result = render_template('edit.html', cfg_form=form, Units=unit)
    return result


@app.route('/mapping/new/', methods=['GET', 'POST'])
def configuration():
    result = None
    form = ConfigSubmitForm()
    result = redirect(url_for('mapping'))
    unit = hwsgcconhandler.GetUnits()

    if form.validate_on_submit():
        name = form.paramProfileName.data.decode('utf-8')
        height = ConvertToMM(float(form.paramHeight.data))
        leftRight = ConvertToMM(float(form.paramLeftRight.data))
        duplicate_name = hwsgcconhandler.IsProfileNameDuplicate('',name)
        height_valid = hwsgcconhandler.ValidateAntennaHeight(height)
        center_offset_valid = hwsgcconhandler.ValidateAntennaLeftRight(abs(leftRight))

        if not center_offset_valid or not height_valid or duplicate_name:
            result = render_template('configuration.html', cfg_form=form,
                                 Units=unit,duplicate = duplicate_name
                                 , invalid_height_offset = not height_valid
                                 , invalid_center_offset = not center_offset_valid)
        else:
            print 'Creating profile ' + name
            hwsgcconhandler.CreateNewProfile(name, 0, height, leftRight)
            hwsgcconhandler.InitializeDifferentialSetting()
    else:
        print form.errors
        print 'render configuration screen'
        result = render_template('configuration.html', cfg_form=form,
                                 Units=unit)
    return result


@app.route('/mapping/setdirection/<direction>')
def setDirection(direction='forward'):
    if direction == 'forward':
        hwsgcconhandler.SetDirection(True)
    else:
        hwsgcconhandler.SetDirection(False)
    print 'direction set to ' + direction
    return redirect(url_for('mapping'))


@app.route('/mapping/', methods=['POST', 'GET'])
@app.route('/mapping/<profile>')
def mapping(profile=''):
    result = None
    direction = hwsgcconhandler.GetData('direction')
    convergence = hwsgcconhandler.GetData('convergence_status')
    if hwsgcconhandler.GetData('num_of_profiles_available') == 5:
        print 'no profiles go to new screen'
        result = redirect(url_for('configuration'))
    else:
        if request.method == 'POST':
            direction = True
            if request.form['direction'] == 'reverse':
                direction = False
            hwsgcconhandler.SetDirection(direction)
        source_list = hwsgcconhandler.GetData('source_list')
        selected_source = hwsgcconhandler.GetData('source')
        if profile != '':
            print profile
            hwsgcconhandler.SetProfile(profile)
        selectedProfile = hwsgcconhandler.GetData('selected_profile')
        listOfProfiles = hwsgcconhandler.GetData('profile_list')
        print selectedProfile
        result = render_template('mapping.html',
                                 profileList=listOfProfiles,
                                 selectedProfile=selectedProfile,
                                 gpsState=convergence,
                                 forward=direction,sourcelist=source_list,gps_source = selected_source)
    return result


@app.route('/mapping/units/<unit>')
def units(unit='in'):
    if 'cm' in unit:
        hwsgcconhandler.SetUnits('Centimeters')
    else:
        hwsgcconhandler.SetUnits('Inches')
    if '-e' in unit:
        result = redirect(url_for('edit'))
    else:
        result = redirect(url_for('configuration'))
    return result

@app.route('/mapping/gpssource/<source>')
def gps_source(source='GL!DE'):
    hwsgcconhandler.SetSource(source)
    result = redirect(url_for('mapping'))
    return result
