#!/bin/bash

CALIBRATOR_PATH=/usr/local/src/selftest
SDIO_MODULE_PATH=/lib/modules/3.10.54-dey+g441c8d4/extra
SDIO_MODULE=wl12xx.ko
INI_FILE=930-0005-R1.2.ini
# FREQ_BAND should be set to single or dual
FREQ_BAND=single
NVS_FILE=/lib/firmware/ti-connectivity/wl1271-nvs.bin
NEW_NVS_FILE=new-nvs.bin
MAC_ADDR=00:00:00:00:00:00
WORKING_DIR=$(pwd)


function check_root {
    if [[ ${UID} -ne 0 ]] ; then
        echo "$0 must be run as sudo user or root"
        exit
    fi
}

check_root

$CALIBRATOR_PATH/calibrator --version |grep "calibrator version 0.74"
if [ $? == 0 ] ; then
	echo "Correct version of calibrator tool"
else
	$CALIBRATOR_PATH/calibrator --version
	echo "Wrong version of calibrator.   Must be 0.74"
	echo
	exit
fi	

echo

#check if NVS file already exists
if [ -f $NVS_FILE ]
   then
      echo "Backing up current NVS file"
      cp $NVS_FILE $NVS_FILE.bak
      echo "removing current NVS file"
      rm $NVS_FILE
fi

#if new NVS exists
if [ -f $NEW_NVS_FILE ]
   then
      echo "removing new NVS file"
      rm $NEW_NVS_FILE
fi

#create reference NVS
echo $CALIBRATOR_PATH/calibrator  set ref_nvs $CALIBRATOR_PATH/$INI_FILE $WORKING_DIR/$NEW_NVS_FILE
$CALIBRATOR_PATH/calibrator  set ref_nvs $CALIBRATOR_PATH/$INI_FILE $WORKING_DIR/$NEW_NVS_FILE

#copy reference NVS to correct location
echo "copying reference NVS to correct location"
cp $NEW_NVS_FILE $NVS_FILE
echo "setting wlan0 down"
ifconfig wlan0 down

#unload current sdio module
echo "unloading wl12xx"
rmmod $SDIO_MODULE

#reload nvs from filesystem
echo "loading wl12xx"
insmod $SDIO_MODULE_PATH/$SDIO_MODULE
echo "setting wlan0 down"
#ifconfig wlan0 down

#echo "Run calibrate"
#./calibrator --debug plt calibrate $FREQ_BAND

sleep 1
echo "plt power_mode on"
$CALIBRATOR_PATH/calibrator  wlan0 plt power_mode on

sleep 1
echo "plt tune_channel 0 7"
$CALIBRATOR_PATH/calibrator  wlan0 plt tune_channel 0 7

sleep 1
echo "plt tx_bip 1 0 0 0 0 0 0 0"
$CALIBRATOR_PATH/calibrator  wlan0 plt tx_bip 1 0 0 0 0 0 0 0


echo "Set MAC addr: " $MAC_ADDR
$CALIBRATOR_PATH/calibrator  set nvs_mac $NEW_NVS_FILE $MAC_ADDR

echo "copy NVS file to correct location"
cp $NEW_NVS_FILE $NVS_FILE

sleep 1
echo "Re-load wl12xx.ko"
rmmod $SDIO_MODULE
insmod $SDIO_MODULE_PATH/$SDIO_MODULE
